@@ -1,7 +1,7 @@
 """ocp_vscode - OCC viewer for VSCode"""
 
 #
-# Copyright 2023 Bernhard Walter
+# Copyright 2025 Bernhard Walter
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,9 +15,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-__version__ = "2.5.1"
+__version__ = "2.9.0"
 
 import os
+import sys
 
 from .show import *
 from .config import *
@@ -25,8 +26,10 @@ from .comms import *
 
 from .colors import *
 from .animation import Animation
+from .selectors import *
 from ocp_tessellate.cad_objects import ImageFace
 
+
 try:
     from ocp_tessellate.tessellator import (
         enable_native_tessellator,
