@@ -1,5 +1,20 @@
 """Color maps for the OCP CAD Viewer"""
 
+#
+# Copyright 2025 Bernhard Walter
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from colorsys import hsv_to_rgb, rgb_to_hsv
 from random import randrange, seed, random
 from webcolors import name_to_rgb
@@ -236,7 +251,7 @@ def matplotlib_mapper(t, name):
         raise ValueError(f"No colormap named '{name}' in matplotlib")
     if not isinstance(colormap, mpl.colors.LinearSegmentedColormap):
         raise ValueError(
-            f"The colormap named '{name}' is not a linear segemented colormap"
+            f"The colormap named '{name}' is not a linear segmented colormap"
         )
 
     color = colormap(t)[:3]
@@ -310,7 +325,7 @@ class ListedColorMap(BaseColorMap):
 
 
 class SegmentedColorMap(BaseColorMap):
-    """A segemented colormap"""
+    """A segmented colormap"""
 
     def __init__(self, length, mapper, alpha=1.0, reverse=False, **params):
         super().__init__()
@@ -476,7 +491,7 @@ class ColorMap:
             _, name = colormap.split(":")
             if not isinstance(mpl.colormaps[name], mpl.colors.LinearSegmentedColormap):
                 raise ValueError(
-                    f"{name} is not a segemented matplotlib colormap,"
+                    f"{name} is not a segmented matplotlib colormap,"
                     f", use ColorMap.listed({length}, {colormap}))"
                 )
             return SegmentedColorMap(
