@@ -1,7 +1,7 @@
 """Configuration of the viewer"""
 
 #
-# Copyright 2023 Bernhard Walter
+# Copyright 2025 Bernhard Walter
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,23 @@
 # limitations under the License.
 #
 
+import os
+
+if os.environ.get("JUPYTER_CADQUERY") is None:
+    from ocp_vscode.comms import send_command, send_config, get_port, is_pytest
+
+    is_jupyter_cadquery = False
+else:
+    from jupyter_cadquery.comms import send_config, send_command
+
+    is_pytest = lambda: False
+
+    is_jupyter_cadquery = True
+
+from ocp_tessellate.utils import Color
+
 from enum import Enum
 
-from .comms import send_command, send_config, get_port, is_pytest
 
 __all__ = [
     "workspace_config",
@@ -54,6 +68,13 @@ class Collapse(Enum):
 
 COLLAPSE_MAPPING = ["E", "1", "C", "R"]
 
+COLLAPSE_REVERSE_MAPPING = {
+    "E": Collapse.NONE,
+    "1": Collapse.LEAVES,
+    "C": Collapse.ALL,
+    "R": Collapse.ROOT,
+}
+
 CONFIG_UI_KEYS = [
     "axes",
     "axes0",
@@ -128,6 +149,7 @@ CONFIG_SET_KEYS = [
     "axes",
     "axes0",
     "grid",
+    "center_grid",
     "ortho",
     "transparent",
     "black_edges",
@@ -170,7 +192,6 @@ DEFAULTS = {
     "show_parent": False,
     "show_sketch_local": True,
     "timeit": False,
-    "reset_camera": Camera.RESET,
     "collapse": Collapse.ROOT,
     "debug": False,
 }
@@ -181,6 +202,7 @@ def set_viewer_config(
     axes=None,
     axes0=None,
     grid=None,
+    center_grid=None,
     ortho=None,
     transparent=None,
     black_edges=None,
@@ -215,21 +237,26 @@ def set_viewer_config(
     clip_planes=None,
     clip_object_colors=None,
     port=None,
+    viewer=None,
 ):
     """Set viewer config"""
-    if port is None:
+    if not is_jupyter_cadquery and port is None:
         port = get_port()
 
     config = {k: v for k, v in locals().items() if v is not None}
+
     if config.get("collapse") is not None:
         config["collapse"] = COLLAPSE_MAPPING[config["collapse"].value]
+    if config.get("default_edgecolor") is not None:
+        config["default_edgecolor"] = Color(config["default_edgecolor"]).web_color
+
     data = {
         "type": "ui",
         "config": config,
     }
 
     try:
-        send_config(data, port)
+        send_config(data, port=port, title=viewer)
 
     except Exception as ex:
         raise RuntimeError(
@@ -239,12 +266,14 @@ def set_viewer_config(
 
 def get_default(key):
     """Get default value for key"""
-    return DEFAULTS.get(key)
+    return get_defaults().get(key)
 
 
 def get_defaults():
     """Get all defaults"""
-    return DEFAULTS
+    result = dict(workspace_config())
+    result.update(DEFAULTS)
+    return result
 
 
 def set_defaults(
@@ -297,6 +326,10 @@ def set_defaults(
     mate_scale=None,  # DEPRECATED
     debug=None,
     timeit=None,
+    # Jupyter CadQuery
+    viewer=None,
+    cad_width=None,
+    height=None,
 ):
     # pylint: disable=line-too-long
     """Set viewer defaults
@@ -329,9 +362,9 @@ def set_defaults(
         position:          Camera position
         quaternion:        Camera orientation as quaternion
         target:            Camera look at target
-        reset_camera:      Camera.RESET: Reset camera position, rotation, toom and target
-                           Camera.CENTER: Keep camera position, rotation, toom, but look at center
-                           Camera.KEEP: Keep camera position, rotation, toom, and target
+        reset_camera:      Camera.RESET: Reset camera position, rotation, zoom and target
+                           Camera.CENTER: Keep camera position, rotation, zoom, but look at center
+                           Camera.KEEP: Keep camera position, rotation, zoom, and target
                            (default=Camera.RESET)
         clip_slider_0:     Setting of clipping slider 0 (default=None)
         clip_slider_1:     Setting of clipping slider 1 (default=None)
@@ -370,6 +403,11 @@ def set_defaults(
     - Debug
         debug:             Show debug statements to the VS Code browser console (default=False)
         timeit:            Show timing information from level 0-3 (default=False)
+
+    - Jupyter Cadquery only:
+        viewer:            The title of the sidecar in Jupyter CadQuery
+        cad_width:         The viewer width in  Jupyter CadQuery
+        height:            The viewer height in  Jupyter CadQuery
     """
 
     kwargs = {k: v for k, v in locals().items() if v is not None}
@@ -377,12 +415,16 @@ def set_defaults(
     kwargs = check_deprecated(kwargs)
 
     for key, value in kwargs.items():
-        if key in CONFIG_KEYS:
+        if key in CONFIG_KEYS or (
+            is_jupyter_cadquery and key in ["viewer", "cad_width", "height"]
+        ):
             DEFAULTS[key] = value
         else:
-            print(f"'{key}' is an unkown config, ignored!")
+            print(f"'{key}' is an unknown config, ignored!")
 
-    set_viewer_config(**{k: v for k, v in kwargs.items() if k in CONFIG_SET_KEYS})
+    set_viewer_config(
+        viewer=viewer, **{k: v for k, v in kwargs.items() if k in CONFIG_SET_KEYS}
+    )
 
 
 def preset(key, value):
@@ -395,20 +437,27 @@ def ui_filter(conf):
     return {k: v for k, v in conf.items() if k in CONFIG_UI_KEYS}
 
 
-def status(port=None, debug=False):
+def workspace_filter(conf):
+    """Filter out all non-workspace keys from the config dict"""
+    return {k: v for k, v in conf.items() if k in CONFIG_WORKSPACE_KEYS}
+
+
+def status(port=None, viewer=None, debug=False):
     """Get viewer status"""
 
     if is_pytest():
         return {}
 
-    if port is None:
+    if not is_jupyter_cadquery and port is None:
         port = get_port()
     try:
-        response = send_command("status", port=port)
+        response = send_command("status", port=port, title=viewer)
         if debug:
             return response.get("_debugStarted", False)
         else:
-            return dict(sorted(response.get("text", {}).items()))
+            if response.get("collapse") is not None:
+                response["collapse"] = COLLAPSE_REVERSE_MAPPING[response["collapse"]]
+            return dict(sorted(response.items()))
 
     except Exception as ex:
         raise RuntimeError(
@@ -416,7 +465,7 @@ def status(port=None, debug=False):
         ) from ex
 
 
-def workspace_config(port=None):
+def workspace_config(port=None, viewer=None):
     """Get viewer workspace config"""
 
     if is_pytest():
@@ -427,10 +476,10 @@ def workspace_config(port=None):
             "default_vertexcolor": (123, 45, 6),
         }
 
-    if port is None:
+    if not is_jupyter_cadquery and port is None:
         port = get_port()
     try:
-        conf = send_command("config", port=port)
+        conf = send_command("config", port=port, title=viewer)
         mapping = {
             "none": Collapse.NONE,
             "leaves": Collapse.LEAVES,
@@ -441,8 +490,11 @@ def workspace_config(port=None):
             "C": Collapse.ALL,
             "R": Collapse.ROOT,
         }
-        conf["collapse"] = mapping[conf.get("collapse", "R")]
-        return dict(sorted(conf.items()))
+        if isinstance(conf.get("collapse"), str):
+            conf["collapse"] = mapping[conf.get("collapse", "R")]
+        if isinstance(conf.get("reset_camera"), str):
+            conf["reset_camera"] = Camera[conf.get("reset_camera", "RESET").upper()]
+        return dict(conf)
 
     except Exception as ex:
         raise RuntimeError(
@@ -450,32 +502,27 @@ def workspace_config(port=None):
         ) from ex
 
 
-def combined_config(port=None, use_status=True):
+def combined_config(port=None, viewer=None):
     """Get combined config from workspace and status"""
-    if port is None:
+
+    if not is_jupyter_cadquery and port is None:
         port = get_port()
 
     try:
-        wspace_config = workspace_config(port)
-        wspace_status = status(port)
+        wspace_config = workspace_config(port=port, viewer=viewer)
+        wspace_status = status(port=port, viewer=viewer)
 
     except Exception as ex:
         raise RuntimeError(
             "Cannot access viewer config. Is the viewer running?\n" + str(ex.args)
         ) from ex
 
-    if use_status and wspace_config["_splash"]:
-        del wspace_config["_splash"]
-        wspace_config["axes"] = False
-        wspace_config["axes0"] = True
-        wspace_config["grid"] = [True, False, False]
-        wspace_config["ortho"] = False
-        wspace_config["transparent"] = False
-        wspace_config["black_edges"] = False
+    use_status = not wspace_config.get("_splash", False)
 
     wspace_config.update(DEFAULTS)
+
     if use_status:
-        wspace_config.update(ui_filter(wspace_status))
+        wspace_config.update(workspace_filter(wspace_status))
     return dict(sorted(wspace_config.items()))
 
 
@@ -511,9 +558,12 @@ def reset_defaults():
         "render_mates": False,
         "render_joints": False,
         "helper_scale": 1.0,
+        "show_parent": False,
+        "show_sketch_local": True,
         "timeit": False,
-        "reset_camera": Camera.RESET,
+        "collapse": Collapse.ROOT,
         "debug": False,
+        # "reset_camera": Camera.RESET,
     }
 
 
@@ -558,4 +608,10 @@ def check_deprecated(kwargs):
         )
         kwargs["collapse"] = Collapse.NONE
 
+    if kwargs.get("control") is not None:
+        print(
+            "\n'control=\"orbit\" or \"trackball\"' is deprecated, use 'orbit_control=True' or 'False' instead\n"
+        )
+        kwargs["orbit_control"] = kwargs["control"] == "orbit"
+
     return kwargs
