@@ -1,7 +1,7 @@
 """Show CAD objects in Visual Studio Code"""
 
 #
-# Copyright 2023 Bernhard Walter
+# Copyright 2025 Bernhard Walter
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,6 +32,7 @@ from ocp_tessellate.cad_objects import (
     OCP_Part,
     OCP_PartGroup,
     OCP_Vertices,
+    OcpWrapper,
 )
 from ocp_tessellate.convert import (
     combined_bb,
@@ -53,26 +54,45 @@ from ocp_tessellate.ocp_utils import (
 from ocp_tessellate.utils import Color, Timer, numpy_to_buffer_json
 
 from ocp_vscode.colors import BaseColorMap, get_colormap
-from ocp_vscode.comms import is_pytest, send_backend, send_command, send_data
+
+if os.environ.get("JUPYTER_CADQUERY") == "1":
+    from jupyter_cadquery.comms import send_backend, send_command, send_data
+
+    is_jupyter_cadquery = True
+
+else:
+    from ocp_vscode.comms import send_backend, send_command, send_data
+
+    is_jupyter_cadquery = False
+
+from ocp_vscode.config import (
+    combined_config,
+    get_changed_config,
+)
+
+from ocp_vscode.comms import is_pytest
 from ocp_vscode.config import (
     Camera,
     Collapse,
     check_deprecated,
-    combined_config,
-    get_changed_config,
     get_default,
     get_defaults,
     preset,
-    workspace_config,
 )
 
 __all__ = [
     "show",
     "show_object",
+    "remove_object",
+    "_show",
+    "_show_object",
+    "push_object",
+    "show_objects",
     "reset_show",
     "show_all",
     "show_clear",
     "save_screenshot",
+    "none_filter",
 ]
 
 OBJECTS = {"objs": [], "names": [], "colors": [], "alphas": []}
@@ -83,11 +103,13 @@ LAST_CALL = "other"
 def _tessellate(
     *cad_objs, names=None, colors=None, alphas=None, progress=None, **kwargs
 ):
-    if workspace_config().get("_splash"):
-        conf = combined_config(use_status=False)
+    viewer = kwargs.get("viewer")
+    port = kwargs.get("port")
+
+    conf = combined_config(viewer=viewer, port=port)
+    if conf.get("_splash") == True:
         reset_camera = Camera.RESET
     else:
-        conf = combined_config(use_status=True)
         reset_camera = conf.get("reset_camera", Camera.RESET)
 
     conf["reset_camera"] = reset_camera.value
@@ -171,35 +193,41 @@ def _tessellate(
                 "show_sketch_local", changed_config.get("show_sketch_local")
             ),
             progress=progress,
+            debug=kwargs.get("debug", False),
         )
 
         if len(part_group.objects) == 1 and isinstance(part_group.objects[0], OcpGroup):
+            loc = part_group.loc
             part_group = part_group.objects[0]
+            part_group.loc = loc * part_group.loc
 
     params = {
         k: v
         for k, v in conf.items()
-        if not k
-        in (
-            "position",
-            "rotation",
-            "target",
-            # controlled by VSCode panel size
-            "cad_width",
-            "height",
-            # controlled by VSCode settings
-            "tree_width",
-            "theme",
+        if not (
+            k in ("position", "rotation", "target")
+            or (
+                not is_jupyter_cadquery
+                and k
+                in (
+                    # controlled by VSCode panel size
+                    "cad_width",
+                    "height",
+                    # controlled by VSCode settings
+                    "tree_width",
+                    "theme",
+                )
+            )
         )
     }
 
     for k, v in kwargs.items():
-        if k in ["cad_width", "height"]:
+        if not is_jupyter_cadquery and k in ["cad_width", "height"]:
             print(
                 f"Setting {k} cannot be set, it is determined by the VSCode panel size"
             )
 
-        elif k in [
+        elif not is_jupyter_cadquery and k in [
             "tree_width",
             "theme",
         ]:
@@ -207,9 +235,13 @@ def _tessellate(
 
         elif v is not None:
             if k == "reset_camera" and params.get("_splash") is True:
+                print(k, v)
+                # do not keep the position and rotation of the splash screen
                 continue
             params[k] = v
 
+    params["_splash"] = False  # after the first show, _splash is False
+
     if kwargs.get("debug") is not None and kwargs["debug"]:
         print("\ntessellation parameters:\n", params)
 
@@ -264,7 +296,7 @@ def _convert(
     if config.get("dark") is not None:
         config["theme"] = "dark"
     elif config.get("orbit_control") is not None:
-        config["control"] = "orbit" if config["control"] else "trackball"
+        config["control"] = "orbit" if config["orbit_control"] else "trackball"
 
     if config.get("debug") is not None and config["debug"]:
         print("\nconfig:\n", config)
@@ -315,6 +347,12 @@ def align_attrs(attr_list, length, default, tag, explode=True):
         return attr_list
 
 
+def none_filter(d, excludes=None):
+    if excludes is None:
+        excludes = []
+    return {k: v for k, v in dict(d).items() if v is not None and k not in excludes}
+
+
 # pylint: disable=unused-argument
 def show(
     *cad_objs,
@@ -334,11 +372,13 @@ def show(
     default_opacity=None,
     black_edges=None,
     orbit_control=None,
+    control=None,
     collapse=None,
     explode=None,
     ticks=None,
     center_grid=None,
     up=None,
+    tab=None,
     zoom=None,
     position=None,
     quaternion=None,
@@ -389,12 +429,12 @@ def show(
         names:                   List of names for the cad_objs. Needs to have the same length as cad_objs
         colors:                  List of colors for the cad_objs. Needs to have the same length as cad_objs
         alphas:                  List of alpha values for the cad_objs. Needs to have the same length as cad_objs
-        port:                    The port the viewer listens to. Typically use 'set_port(port)' instead
         progress:                Show progress of tessellation with None is no progress indicator. (default="-+*c")
                                  for object: "-": is reference,
                                              "+": gets tessellated with Python code,
                                              "*": gets tessellated with native code,
                                              "c": from cache
+        port:                    The port the viewer listens to. Typically use 'set_port(port)' instead
 
     Valid keywords to configure the viewer (**kwargs):
     - UI
@@ -425,9 +465,9 @@ def show(
         position:                Camera position
         quaternion:              Camera orientation as quaternion
         target:                  Camera look at target
-        reset_camera:            Camera.RESET: Reset camera position, rotation, toom and target
-                                 Camera.CENTER: Keep camera position, rotation, toom, but look at center
-                                 Camera.KEEP: Keep camera position, rotation, toom, and target
+        reset_camera:            Camera.RESET: Reset camera position, rotation, zoom and target
+                                 Camera.CENTER: Keep camera position, rotation, zoom, but look at center
+                                 Camera.KEEP: Keep camera position, rotation, zoom, and target
                                  (default=Camera.RESET)
 
         clip_slider_0:           Setting of clipping slider 0 (default=None)
@@ -471,8 +511,22 @@ def show(
         debug:                   Show debug statements to the VS Code browser console (default=False)
         timeit:                  Show timing information from level 0-3 (default=False)
     """
+
+    return _show(*cad_objs, **none_filter(locals(), ["cad_objs"]))
+
+
+def _show(*cad_objs, **kwargs):
     global LAST_CALL  # pylint: disable=global-statement
 
+    port = kwargs.get("port")
+    timeit = kwargs.get("timeit")
+    names = kwargs.get("names")
+    colors = kwargs.get("colors")
+    alphas = kwargs.get("alphas")
+    default_edgecolor = kwargs.get("default_edgecolor")
+    progress = kwargs.get("progress")
+    _force_in_debug = kwargs.get("_force_in_debug")
+
     if (
         cad_objs is None
         or len(cad_objs) == 0
@@ -492,7 +546,7 @@ def show(
 
     kwargs = {
         k: v
-        for k, v in locals().items()
+        for k, v in kwargs.items()
         if v is not None
         and k
         not in [
@@ -538,7 +592,7 @@ def show(
                     colors[i].a = alphas[i]
 
             if hasattr(cad_objs[i], "color") and cad_objs[i].color is not None:
-                # ensure that the explicitely given color is kept
+                # ensure that the explicitly given color is kept
                 colors[i] = Color(cad_objs[i].color)
         else:
             colors[i] = Color(colors[i])
@@ -566,13 +620,20 @@ def show(
         else:
             LAST_CALL = "other"
 
+    if progress is not None:
+        print()
+
     if is_pytest():
         return t, mapping
 
     with Timer(timeit, "", "send"):
-        send_data(t, port=port, timeit=timeit)
+        viewer = send_data(t, port=port, timeit=timeit)
 
-    send_backend({"model": mapping}, port=port, timeit=timeit)
+    if is_jupyter_cadquery:
+        send_backend({"model": mapping}, jcv_id=viewer.widget.id, timeit=timeit)
+        return viewer
+    else:
+        send_backend({"model": mapping}, port=port, timeit=timeit)
 
 
 def reset_show():
@@ -589,6 +650,7 @@ def show_object(
     options=None,
     parent=None,
     clear=False,
+    update=False,
     port=None,
     progress="-+*c",
     glass=None,
@@ -657,8 +719,9 @@ def show_object(
         options:                 A dict of color and alpha value: {"alpha":0.5, "color": (64, 164, 223)}
                                  0 <= alpha <= 1.0 and color is a 3-tuple of values between 0 and 255
         parent:                  Add another object, usually the parent of e.g. edges or vertices with alpha=0.25
-        clear:                   In interactice mode, clear the stack of objects to be shown
+        clear:                   In interactive mode, clear the stack of objects to be shown
                                  (typically used for the first object)
+        update:                  Update the object (remove old version)
         port:                    The port the viewer listens to. Typically use 'set_port(port)' instead
         progress:                Show progress of tessellation with None is no progress indicator. (default="-+*c")
                                  for object: "-": is reference,
@@ -695,9 +758,9 @@ def show_object(
         position:                Camera position
         quaternion:              Camera orientation as quaternion
         target:                  Camera look at target
-        reset_camera:            Camera.RESET: Reset camera position, rotation, toom and target
-                                 Camera.CENTER: Keep camera position, rotation, toom, but look at center
-                                 Camera.KEEP: Keep camera position, rotation, toom, and target
+        reset_camera:            Camera.RESET: Reset camera position, rotation, zoom and target
+                                 Camera.CENTER: Keep camera position, rotation, zoom, but look at center
+                                 Camera.KEEP: Keep camera position, rotation, zoom, and target
                                  (default=Camera.RESET)
 
         clip_slider_0:           Setting of clipping slider 0 (default=None)
@@ -740,19 +803,63 @@ def show_object(
 
     - Debug
         debug:                   Show debug statements to the VS Code browser console (default=False)
-        imeit:                   Show timing information from level 0-3 (default=False)
+        timeit:                  Show timing information from level 0-3 (default=False)
     """
+    return _show_object(obj, **none_filter(locals(), ["obj"]))
+
+
+def remove_object(name, call_show=False, port=None, progress="-+*c"):
+    """Remove object from the stack of objects by name"""
+    try:
+        index = OBJECTS["names"].index(name)
+        for key in ["objs", "names", "colors", "alphas"]:
+            del OBJECTS[key][index]
+    except ValueError:
+        pass  # Name not found; silently do nothing
+
+    if call_show:
+        return show(
+            *OBJECTS["objs"],
+            names=OBJECTS["names"],
+            colors=OBJECTS["colors"],
+            alphas=OBJECTS["alphas"],
+            port=port,
+            progress=progress,
+        )
+
+
+def _show_object(obj, **kwargs):
+    port = kwargs.get("port")
+    name = kwargs.get("name")
+    clear = kwargs.get("clear")
+    update = kwargs.get("update")
+    parent = kwargs.get("parent")
+    options = kwargs.get("options")
+    progress = kwargs.get("progress")
 
     kwargs = {
         k: v
-        for k, v in locals().items()
+        for k, v in kwargs.items()
         if v is not None
-        and k not in ["obj", "name", "options", "parent", "clear", "port", "progress"]
+        and k
+        not in [
+            "obj",
+            "name",
+            "options",
+            "parent",
+            "clear",
+            "port",
+            "progress",
+            "update",
+        ]
     }
 
     if clear:
         reset_show()
 
+    if update:
+        remove_object(name)
+
     if parent is not None:
         OBJECTS["objs"].append(parent)
         OBJECTS["names"].append("parent")
@@ -775,7 +882,7 @@ def show_object(
     OBJECTS["colors"].append(color)
     OBJECTS["alphas"].append(alpha)
 
-    show(
+    return show(
         *OBJECTS["objs"],
         names=OBJECTS["names"],
         colors=OBJECTS["colors"],
@@ -786,6 +893,211 @@ def show_object(
     )
 
 
+def push_object(obj, name=None, color=None, alpha=None, clear=False, update=False):
+    """
+    Adds or updates an object in the global OBJECTS registry with optional name, color, and alpha
+    transparency.
+
+    Parameters:
+        obj: The object to be added or updated. Must have 'name', 'label', 'color', or 'alpha'
+            ttributes if corresponding arguments are not provided.
+        name (str, optional): The name to associate with the object. If not provided,
+            attempts to use 'name' or 'label' attribute of obj.
+        color (any, optional): The color to associate with the object. If not provided,
+            attempts to use 'color' attribute of obj.
+        alpha (float, optional): The alpha (transparency) value for the object. If not provided,
+            attempts to use 'alpha' attribute of obj, defaults to 1.0.
+        clear (bool, optional): If True, clears the OBJECTS registry before adding the new object.
+        update (bool, optional): If True, updates an existing object with the same name;
+            otherwise, appends as a new object.
+    Raises:
+        ValueError: If no name is provided and the object does not have a 'name' or 'label' attribute.
+    """
+    if clear:
+        reset_show()
+
+    if name is None:
+        if hasattr(obj, "name"):
+            name = obj.name
+        elif hasattr(obj, "label"):
+            name = obj.label
+        else:
+            raise ValueError("No name provided and no name attribute found.")
+    if color is None:
+        if hasattr(obj, "color"):
+            color = obj.color
+    if alpha is None:
+        if hasattr(obj, "alpha"):
+            alpha = obj.alpha
+        else:
+            alpha = 1.0
+
+    if update:
+        index = OBJECTS["names"].index(name)
+        OBJECTS["objs"][index] = obj
+        OBJECTS["colors"][index] = color
+        OBJECTS["alphas"][index] = alpha
+    else:
+        OBJECTS["objs"].append(obj)
+        OBJECTS["names"].append(name)
+        OBJECTS["colors"].append(color)
+        OBJECTS["alphas"].append(alpha)
+
+
+def show_objects(
+    port=None,
+    progress="-+*c",
+    glass=None,
+    tools=None,
+    tree_width=None,
+    axes=None,
+    axes0=None,
+    grid=None,
+    ortho=None,
+    transparent=None,
+    default_opacity=None,
+    black_edges=None,
+    orbit_control=None,
+    collapse=None,
+    ticks=None,
+    center_grid=None,
+    up=None,
+    zoom=None,
+    position=None,
+    quaternion=None,
+    target=None,
+    reset_camera=None,
+    clip_slider_0=None,
+    clip_slider_1=None,
+    clip_slider_2=None,
+    clip_normal_0=None,
+    clip_normal_1=None,
+    clip_normal_2=None,
+    clip_intersection=None,
+    clip_planes=None,
+    clip_object_colors=None,
+    pan_speed=None,
+    rotate_speed=None,
+    zoom_speed=None,
+    deviation=None,
+    angular_tolerance=None,
+    edge_accuracy=None,
+    default_color=None,
+    default_facecolor=None,
+    default_thickedgecolor=None,
+    default_vertexcolor=None,
+    default_edgecolor=None,
+    ambient_intensity=None,
+    metalness=None,
+    roughness=None,
+    direct_intensity=None,
+    render_edges=None,
+    render_normals=None,
+    render_mates=None,
+    render_joints=None,
+    show_parent=None,
+    show_sketch_local=None,
+    helper_scale=None,
+    mate_scale=None,  # DEPRECATED
+    debug=None,
+    timeit=None,
+):
+    # pylint: disable=line-too-long
+    """Show incrementally pushed CAD objects in Visual Studio Code
+
+    Keywords for show_objects:
+        progress:                Show progress of tessellation with None is no progress indicator. (default="-+*c")
+                                 for object: "-": is reference,
+                                             "+": gets tessellated with Python code,
+                                             "*": gets tessellated with native code,
+                                             "c": from cache
+
+
+    Valid keywords to configure the viewer (**kwargs):
+    - UI
+        glass:                   Use glass mode where tree is an overlay over the cad object (default=False)
+        tools:                   Show tools (default=True)
+        tree_width:              Width of the object tree (default=240)
+
+    - Viewer
+        axes:                    Show axes (default=False)
+        axes0:                   Show axes at (0,0,0) (default=False)
+        grid:                    Show grid (default=False)
+        ortho:                   Use orthographic projections (default=True)
+        transparent:             Show objects transparent (default=False)
+        default_opacity:         Opacity value for transparent objects (default=0.5)
+        black_edges:             Show edges in black color (default=False)
+        orbit_control:           Mouse control use "orbit" control instead of "trackball" control (default=False)
+        collapse:                Collapse.LEAVES: collapse all single leaf nodes,
+                                 Collapse.ROOT: expand root only,
+                                 Collapse.ALL: collapse all nodes,
+                                 Collapse.NONE: expand all nodes
+                                 (default=Collapse.ROOT)
+        ticks:                   Hint for the number of ticks in both directions (default=10)
+        center_grid:             Center the grid at the origin or center of mass (default=False)
+        up:                      Use z-axis ('Z') or y-axis ('Y') as up direction for the camera (default="Z")
+
+        zoom:                    Zoom factor of view (default=1.0)
+        position:                Camera position
+        quaternion:              Camera orientation as quaternion
+        target:                  Camera look at target
+        reset_camera:            Camera.RESET: Reset camera position, rotation, zoom and target
+                                 Camera.CENTER: Keep camera position, rotation, zoom, but look at center
+                                 Camera.KEEP: Keep camera position, rotation, zoom, and target
+                                 (default=Camera.RESET)
+
+        clip_slider_0:           Setting of clipping slider 0 (default=None)
+        clip_slider_1:           Setting of clipping slider 1 (default=None)
+        clip_slider_2:           Setting of clipping slider 2 (default=None)
+        clip_normal_0:           Setting of clipping normal 0 (default=[-1,0,0])
+        clip_normal_1:           Setting of clipping normal 1 (default=[0,-1,0])
+        clip_normal_2:           Setting of clipping normal 2 (default=[0,0,-1])
+        clip_intersection:       Use clipping intersection mode (default=[False])
+        clip_planes:             Show clipping plane helpers (default=False)
+        clip_object_colors:      Use object color for clipping caps (default=False)
+
+        pan_speed:               Speed of mouse panning (default=1)
+        rotate_speed:            Speed of mouse rotate (default=1)
+        zoom_speed:              Speed of mouse zoom (default=1)
+
+    - Renderer
+        deviation:               Shapes: Deviation from linear deflection value (default=0.1)
+        angular_tolerance:       Shapes: Angular deflection in radians for tessellation (default=0.2)
+        edge_accuracy:           Edges: Precision of edge discretization (default: mesh quality / 100)
+
+        default_color:           Default mesh color (default=(232, 176, 36))
+        default_edgecolor:       Default color of the edges of a mesh (default=(128, 128, 128))
+        default_facecolor:       Default color of the edges of a mesh (default=#ee82ee / Violet)
+        default_thickedgecolor:  Default color of the edges of a mesh (default=#ba55d3 / MediumOrchid)
+        default_vertexcolor:     Default color of the edges of a mesh (default=#ba55d3 / MediumOrchid)
+        ambient_intensity:       Intensity of ambient light (default=1.00)
+        direct_intensity:        Intensity of direct light (default=1.10)
+        metalness:               Metalness property of the default material (default=0.30)
+        roughness:               Roughness property of the default material (default=0.65)
+
+
+        render_edges:            Render edges  (default=True)
+        render_normals:          Render normals (default=False)
+        render_mates:            Render mates for MAssemblies (default=False)
+        render_joints:           Render build123d joints (default=False)
+        show_parent:             Render parent of faces, edges or vertices as wireframe (default=False)
+        show_sketch_local:       In build123d show local sketch in addition to relocate sketch (default=True)
+        helper_scale:            Scale of rendered helpers (locations, axis, mates for MAssemblies) (default=1)
+
+    - Debug
+        debug:                   Show debug statements to the VS Code browser console (default=False)
+        timeit:                  Show timing information from level 0-3 (default=False)
+    """
+    kwargs = none_filter(locals())
+    return show(
+        *OBJECTS["objs"],
+        names=OBJECTS["names"],
+        colors=OBJECTS["colors"],
+        alphas=OBJECTS["alphas"],
+        **kwargs,
+    )
+
+
 def show_clear():
     """Clear the viewer"""
     data = {
@@ -798,10 +1110,24 @@ def show_all(
     variables=None,
     exclude=None,
     classes=None,
+    include=None,
     _visual_debug=False,
     **kwargs,
 ):
-    """Show all variables in the current scope"""
+    """
+    Show all variables in the current scope
+
+    Parameters:
+        variables:     Only show objects with names in this list of variable names,
+                       i.e. do not use all from locals()
+        exclude:       List of variable names to exclude from "show_all"
+        classes:       Only show objects which are instances of the classes in this list
+        include:       List of variable names to additionally include in "show_all" when classes is None
+        _visual_debug: private variable, do not use!
+
+    Keywords for show_all:
+        Valid keywords for "show_all" are the same as for "show"
+    """
     import inspect  # pylint: disable=import-outside-toplevel
 
     global LAST_CALL  # pylint: disable=global-statement
@@ -829,7 +1155,8 @@ def show_all(
             isinstance(obj, type)
             or name in exclude + ["_", "__", "___", "_ih", "_oh", "_dh", "Out", "In"]
             or name.startswith("__")
-            or re.search("_\\d+", name) is not None
+            or re.search("^_i\\d+", name) is not None
+            or re.search("^_\\d+", name) is not None
             # pylint: disable=protected-access
             or (hasattr(obj, "_obj") and obj._obj is None)
             or callable(obj)
@@ -841,6 +1168,7 @@ def show_all(
                 and obj.__class__.__module__ == "__future__"
             )
             or isinstance(obj, Logger)
+            or "cad_viewer_widget.widget" in str(obj.__class__)
         ):
             continue
 
@@ -887,15 +1215,29 @@ def show_all(
                 names.append(name)
 
             elif is_cadquery_sketch(obj):
-                pg = to_ocpgroup([obj], names=[name])
+                pg, instances = to_ocpgroup([obj], names=[name])
                 pg.name = name
-                objects.append(pg)
+                objects.append(instances)
+                names.append(name)
+
+            elif isinstance(obj, OcpWrapper):
+                objects.append(obj)
                 names.append(name)
 
             else:
-                print(
-                    f"show_all: Type {type(obj)} for name {name} cannot be visualized"
-                )
+                if kwargs.get("debug", False):
+                    print(
+                        f"show_all: Type {type(obj)} for name {name} cannot be visualized"
+                    )
+
+        if (
+            classes is not None
+            and include is not None
+            and isinstance(include, (tuple, list))
+        ):
+            if name in include and name not in names:
+                objects.append(variables[name])
+                names.append(name)
 
     if len(objects) > 0:
         try:
@@ -906,7 +1248,8 @@ def show_all(
                 _force_in_debug=_visual_debug,
                 **kwargs,
             )
-            if is_pytest():
+
+            if is_pytest() or is_jupyter_cadquery:
                 return result
         except Exception as ex:  # pylint: disable=broad-exception-caught
             print("show_all:", ex)
@@ -916,7 +1259,7 @@ def show_all(
         show_clear()
 
 
-def save_screenshot(filename, port=None):
+def save_screenshot(filename, port=None, polling=True):
     """Save a screenshot of the current view"""
     if not filename.startswith(os.sep):
         prefix = pathlib.Path(".").absolute()
@@ -928,13 +1271,14 @@ def save_screenshot(filename, port=None):
 
     send_command({"type": "screenshot", "filename": f"{full_path}"}, port=port)
 
-    done = False
-    for i in range(20):
-        if p.exists() and p.stat().st_mtime > mtime:
-            print("Screenshot saved to ", full_path)
-            done = True
-            break
-        time.sleep(0.1)
+    if polling:
+        done = False
+        for i in range(20):
+            if p.exists() and p.stat().st_mtime > mtime:
+                print("Screenshot saved to ", full_path)
+                done = True
+                break
+            time.sleep(0.1)
 
-    if not done:
-        "Screenshot not found in 2 seconds, aborting"
+        if not done:
+            print("Warning: Screenshot not found in 2 seconds, aborting")
